@@ -13,6 +13,7 @@
 # limitations under the License.
 """Impact analysis."""
 
+import codecs
 import collections
 import logging
 import os
@@ -38,8 +39,14 @@ AnalyzeResult = collections.namedtuple('AnalyzeResult', 'has_changes commits')
 
 TagsInfo = collections.namedtuple('TagsInfo', 'tags latest_tag')
 
-_DATASTORE_BATCH_SIZE = 10000
-_DATASTORE_BATCH_SLEEP = 5
+# Limit for writing small entities.
+_DATASTORE_BATCH_SIZE = 5000
+# Limit for writing large entities that are close to the 1MiB max entity size.
+# There is additionally a request size limit of 10MiB for batched put()
+# requests. We conservatively set this to slightly under (8 as opposed to 10)
+# to leave some more breathing room.
+_DATASTORE_LARGE_BATCH_SIZE = 8
+_DATASTORE_BATCH_SLEEP = 10
 
 
 class ImpactError(Exception):
@@ -321,43 +328,74 @@ def _batcher(entries, batch_size):
     yield entries[i:i + batch_size], i + batch_size >= len(entries)
 
 
-def _throttled_put(to_put):
+def _throttled_put(to_put, batch_size=_DATASTORE_BATCH_SIZE):
   """Throttled ndb put."""
-  for batch, is_last in _batcher(to_put, _DATASTORE_BATCH_SIZE):
+  for batch, is_last in _batcher(to_put, batch_size):
     ndb.put_multi(batch)
     if not is_last:
       time.sleep(_DATASTORE_BATCH_SLEEP)
 
 
-def _throttled_delete(to_delete):
+def _throttled_delete(to_delete, batch_size=_DATASTORE_BATCH_SIZE):
   """Throttled ndb delete."""
-  for batch, is_last in _batcher(to_delete, _DATASTORE_BATCH_SIZE):
+  for batch, is_last in _batcher(to_delete, batch_size):
     ndb.delete_multi(batch)
     if not is_last:
       time.sleep(_DATASTORE_BATCH_SLEEP)
 
 
-def update_affected_commits(bug_id, commits, public):
+def update_affected_commits(bug_id, commits, public, write_legacy=True):
   """Update affected commits."""
   to_put = []
   to_delete = []
 
-  for commit in commits:
-    affected_commit = models.AffectedCommit(
-        id=bug_id + '-' + commit, bug_id=bug_id, commit=commit, public=public)
+  # TODO(ochang): Remove this completedly once AffectedCommits migration is
+  # done.
+  if write_legacy:
+    for commit in commits:
+      affected_commit = models.AffectedCommit(
+          id=bug_id + '-' + commit, bug_id=bug_id, commit=commit, public=public)
 
-    to_put.append(affected_commit)
+      to_put.append(affected_commit)
 
-  # Delete any affected commits that no longer apply. This can happen in cases
-  # where a FixResult comes in later and we had previously marked a commit prior
-  # to the fix commit as being affected by a vulnerability.
-  for existing in models.AffectedCommit.query(
-      models.AffectedCommit.bug_id == bug_id):
-    if existing.commit not in commits:
+    # Delete any affected commits that no longer apply. This can happen in
+    # cases where a FixResult comes in later and we had previously marked a
+    # commit prior to the fix commit as being affected by a vulnerability.
+    for existing in models.AffectedCommit.query(
+        models.AffectedCommit.bug_id == bug_id):
+      if existing.commit not in commits:
+        to_delete.append(existing.key)
+
+    _throttled_put(to_put)
+    _throttled_delete(to_delete)
+
+    to_put.clear()
+    to_delete.clear()
+
+  # Write batched commit indexes.
+  # Sort the commits for ordering consistency in tests.
+  num_pages = 0
+  for batch, _ in _batcher(
+      sorted(commits), models.AffectedCommits.MAX_COMMITS_PER_ENTITY):
+    affected_commits = models.AffectedCommits(
+        id=f'{bug_id}-{num_pages}',
+        bug_id=bug_id,
+        public=public,
+        page=num_pages)
+    affected_commits.commits = [
+        codecs.decode(commit, 'hex') for commit in batch
+    ]
+    to_put.append(affected_commits)
+    num_pages += 1
+
+  # Clear any previously written pages above our current page count.
+  for existing in models.AffectedCommits.query(
+      models.AffectedCommits.bug_id == bug_id):
+    if existing.page >= num_pages:
       to_delete.append(existing.key)
 
-  _throttled_put(to_put)
-  _throttled_delete(to_delete)
+  _throttled_put(to_put, batch_size=_DATASTORE_LARGE_BATCH_SIZE)
+  _throttled_delete(to_delete, batch_size=_DATASTORE_LARGE_BATCH_SIZE)
 
 
 def enumerate_versions(package, ecosystem, affected_range):
@@ -373,7 +411,7 @@ def enumerate_versions(package, ecosystem, affected_range):
       zero_event = event
       continue
 
-    if event.introduced or event.fixed:
+    if event.introduced or event.fixed or event.last_affected:
       sorted_events.append(event)
       continue
 
@@ -384,8 +422,12 @@ def enumerate_versions(package, ecosystem, affected_range):
     """Sort key."""
     if event.introduced:
       return ecosystem.sort_key(event.introduced)
+    if event.fixed:
+      return ecosystem.sort_key(event.fixed)
+    if event.last_affected:
+      return ecosystem.sort_key(event.last_affected)
 
-    return ecosystem.sort_key(event.fixed)
+    raise ValueError('Invalid event')
 
   sorted_events.sort(key=sort_key)
   if zero_event:
@@ -398,14 +440,24 @@ def enumerate_versions(package, ecosystem, affected_range):
 
     if last_introduced and event.fixed:
       current_versions = ecosystem.enumerate_versions(
-          package, last_introduced, event.fixed, limits=limits)
+          package, last_introduced, fixed=event.fixed, limits=limits)
+      if current_versions:
+        versions.update(current_versions)
+      last_introduced = None
+
+    if last_introduced and event.last_affected:
+      current_versions = ecosystem.enumerate_versions(
+          package,
+          last_introduced,
+          last_affected=event.last_affected,
+          limits=limits)
       if current_versions:
         versions.update(current_versions)
       last_introduced = None
 
   if last_introduced:
     current_versions = ecosystem.enumerate_versions(
-        package, last_introduced, None, limits=limits)
+        package, last_introduced, limits=limits)
     if current_versions:
       versions.update(current_versions)
 
@@ -417,7 +469,6 @@ def enumerate_versions(package, ecosystem, affected_range):
 def _analyze_git_ranges(repo_analyzer, checkout_path, affected_range,
                         new_versions, commits, new_introduced, new_fixed):
   """Analyze git ranges."""
-  package_repo_dir = tempfile.TemporaryDirectory()
   package_repo = None
 
   with tempfile.TemporaryDirectory() as package_repo_dir:
@@ -447,6 +498,7 @@ def _analyze_git_ranges(repo_analyzer, checkout_path, affected_range,
         continue
 
     try:
+      # TODO(ochang): Support last_affected.
       result = repo_analyzer.get_affected(package_repo, all_introduced,
                                           all_fixed, all_limit)
     except ImpactError:
@@ -478,7 +530,8 @@ def analyze(vulnerability,
   for affected_package in vulnerability.affected:
     versions = []
     for affected_range in affected_package.ranges:
-      if affected_package.package.ecosystem in ecosystems.SEMVER_ECOSYSTEMS:
+      if (affected_range.type == vulnerability_pb2.Range.ECOSYSTEM and
+          affected_package.package.ecosystem in ecosystems.SEMVER_ECOSYSTEMS):
         # Replace erroneous range type.
         affected_range.type = vulnerability_pb2.Range.SEMVER
 
@@ -486,7 +539,7 @@ def analyze(vulnerability,
                                  vulnerability_pb2.Range.SEMVER):
         # Enumerate ECOSYSTEM and SEMVER ranges.
         ecosystem_helpers = ecosystems.get(affected_package.package.ecosystem)
-        if ecosystem_helpers:
+        if ecosystem_helpers and ecosystem_helpers.supports_ordering:
           try:
             versions.extend(
                 enumerate_versions(affected_package.package.name,
