@@ -30,7 +30,7 @@ from . import semver_index
 from . import sources
 from . import vulnerability_pb2
 
-SCHEMA_VERSION = '1.2.0'
+SCHEMA_VERSION = '1.3.0'
 
 
 def _check_valid_severity(prop, value):
@@ -53,7 +53,7 @@ def _check_valid_event_type(prop, value):
   """Check valid event type."""
   del prop
 
-  if value not in ('introduced', 'fixed', 'limit'):
+  if value not in ('introduced', 'fixed', 'last_affected', 'limit'):
     raise ValueError('Invalid event type: ' + value)
 
 
@@ -62,6 +62,17 @@ def utcnow():
   return datetime.datetime.utcnow()
 
 
+def _get_purl_indexes(affected_packages):
+  """Get list of purls from affected packages, with and without qualifiers"""
+  resulting_set = set()
+  for pkg in affected_packages:
+    if pkg.package.purl:
+      resulting_set.add(pkg.package.purl)
+      if '?' in pkg.package.purl:
+        resulting_set.add(pkg.package.purl.split('?')[0])
+  return list(resulting_set)
+
+
 class IDCounter(ndb.Model):
   """Counter for ID allocations."""
   # Next ID to allocate.
@@ -82,6 +93,20 @@ class AffectedCommit(ndb.Model):
   public = ndb.BooleanProperty()
 
 
+class AffectedCommits(ndb.Model):
+  """AffectedCommits entry."""
+  MAX_COMMITS_PER_ENTITY = 10000
+
+  # The main bug ID.
+  bug_id = ndb.StringProperty()
+  # The commit hash.
+  commits = ndb.BlobProperty(repeated=True, indexed=True)
+  # Whether or not the bug is public.
+  public = ndb.BooleanProperty()
+  # The page for this batch of commits.
+  page = ndb.IntegerProperty(indexed=False)
+
+
 class RegressResult(ndb.Model):
   """Regression results."""
   # The commit hash.
@@ -134,18 +159,6 @@ class FixResult(ndb.Model):
   timestamp = ndb.DateTimeProperty()
 
 
-class AffectedRange(ndb.Model):
-  """Affected range."""
-  # Type of range.
-  type = ndb.StringProperty(validator=_check_valid_range_type)
-  # Repo URL.
-  repo_url = ndb.StringProperty()
-  # The regressing commit.
-  introduced = ndb.StringProperty()
-  # The fix commit.
-  fixed = ndb.StringProperty()
-
-
 class AffectedEvent(ndb.Model):
   """Affected event."""
   type = ndb.StringProperty(validator=_check_valid_event_type)
@@ -223,6 +236,8 @@ class Bug(ndb.Model):
   timestamp = ndb.DateTimeProperty()
   # When the entry was last edited.
   last_modified = ndb.DateTimeProperty()
+  # Last modified field of the original imported file
+  import_last_modified = ndb.DateTimeProperty()
   # When the entry was withdrawn.
   withdrawn = ndb.DateTimeProperty()
   # The source identifier.
@@ -233,8 +248,6 @@ class Bug(ndb.Model):
   fixed = ndb.StringProperty(default='')
   # The main regressing commit (from bisection).
   regressed = ndb.StringProperty(default='')
-  # All affected ranges. TODO(ochang): To be removed.
-  affected_ranges = ndb.StructuredProperty(AffectedRange, repeated=True)
   # List of affected versions.
   affected = ndb.TextProperty(repeated=True)
   # List of normalized versions indexed for fuzzy matching.
@@ -319,33 +332,37 @@ class Bug(ndb.Model):
     value_lower = value.lower()
     return re.split(r'\W+', value_lower) + [value_lower]
 
-  def _pre_put_hook(self):
+  def _pre_put_hook(self):  # pylint: disable=arguments-differ
     """Pre-put hook for populating search indices."""
     search_indices = set()
 
     search_indices.update(self._tokenize(self.id()))
 
-    for affected_package in self.affected_packages:
+    for pkg in self.affected_packages:
       # Set PURL if it wasn't provided.
-      if not affected_package.package.purl:
-        affected_package.package.purl = purl_helpers.package_to_purl(
-            affected_package.package.ecosystem, affected_package.package.name)
+      if not pkg.package.purl:
+        pkg.package.purl = purl_helpers.package_to_purl(
+            ecosystems.normalize(pkg.package.ecosystem), pkg.package.name)
 
     self.project = list({
         pkg.package.name for pkg in self.affected_packages if pkg.package.name
     })
     self.project.sort()
 
-    self.ecosystem = list({
+    ecosystems_set = {
         pkg.package.ecosystem
         for pkg in self.affected_packages
         if pkg.package.ecosystem
-    })
+    }
+
+    # For all ecosystems that specify a specific version with colon,
+    # also add the base name
+    ecosystems_set.update({ecosystems.normalize(x) for x in ecosystems_set})
+
+    self.ecosystem = list(ecosystems_set)
     self.ecosystem.sort()
 
-    self.purl = list({
-        pkg.package.purl for pkg in self.affected_packages if pkg.package.purl
-    })
+    self.purl = _get_purl_indexes(self.affected_packages)
     self.purl.sort()
 
     for project in self.project:
@@ -367,7 +384,8 @@ class Bug(ndb.Model):
 
     for affected_package in self.affected_packages:
       # Indexes used for querying by exact version.
-      if ecosystems.get(affected_package.package.ecosystem):
+      ecosystem_helper = ecosystems.get(affected_package.package.ecosystem)
+      if ecosystem_helper and ecosystem_helper.supports_ordering:
         # No need to normalize if the ecosystem is supported.
         self.affected_fuzzy.extend(affected_package.versions)
       else:
@@ -380,6 +398,10 @@ class Bug(ndb.Model):
         fixed_version = None
         for event in affected_range.events:
           # Index used to query by fixed/unfixed.
+          if event.type == 'limit':
+            self.is_fixed = True
+            fixed_version = event.value
+
           if event.type == 'fixed':
             self.is_fixed = True
             fixed_version = event.value
@@ -468,6 +490,11 @@ class Bug(ndb.Model):
                 AffectedEvent(type='fixed', value=evt.fixed))
             continue
 
+          if evt.last_affected:
+            current_range.events.append(
+                AffectedEvent(type='last_affected', value=evt.last_affected))
+            continue
+
           if evt.limit:
             current_range.events.append(
                 AffectedEvent(type='limit', value=evt.limit))
@@ -504,9 +531,18 @@ class Bug(ndb.Model):
       self.database_specific = json_format.MessageToDict(
           vulnerability.database_specific, preserving_proto_field_name=True)
 
+  def to_vulnerability_minimal(self):
+    """Convert to Vulnerability proto (minimal)."""
+    if self.last_modified:
+      modified = timestamp_pb2.Timestamp()
+      modified.FromDatetime(self.last_modified)
+    else:
+      modified = None
+
+    return vulnerability_pb2.Vulnerability(id=self.id(), modified=modified)
+
   def to_vulnerability(self, include_source=False):
     """Convert to Vulnerability proto."""
-    package = None
     affected = []
 
     source_link = None
@@ -596,7 +632,6 @@ class Bug(ndb.Model):
         withdrawn=withdrawn,
         summary=self.summary,
         details=details,
-        package=package,
         affected=affected,
         severity=severity,
         credits=credits_,
@@ -608,6 +643,44 @@ class Bug(ndb.Model):
     return result
 
 
+class RepoIndex(ndb.Model):
+  """RepoIndex entry"""
+  # The dependency name
+  name = ndb.StringProperty()
+  # The base cpe without the version
+  base_cpe = ndb.StringProperty()
+  # The repository commit
+  commit = ndb.BlobProperty()
+  # Number of hash pages
+  pages = ndb.IntegerProperty()
+  # The source address
+  repo_addr = ndb.StringProperty()
+  # The identified version
+  version = ndb.StringProperty()
+  # The scanned file extensions
+  file_exts = ndb.StringProperty(repeated=True)
+  # The hash algorithm used
+  file_hash_type = ndb.StringProperty()
+  # The repository type
+  repo_type = ndb.StringProperty()
+
+
+class FileResult(ndb.Model):
+  """FileResult entry containing the path and hash"""
+  # The hash value of the file
+  hash = ndb.BlobProperty(indexed=True)
+  # The file path
+  path = ndb.TextProperty()
+
+
+class RepoIndexResult(ndb.Model):
+  """RepoIndexResult entries containing the actual hash values"""
+  # The file results per file
+  file_results = ndb.StructuredProperty(FileResult, repeated=True)
+  # The actual page
+  page = ndb.IntegerProperty()
+
+
 class SourceRepositoryType(enum.IntEnum):
   """SourceRepository type."""
   GIT = 0
@@ -648,6 +721,8 @@ class SourceRepository(ndb.Model):
   detect_cherrypicks = ndb.BooleanProperty(default=True)
   # Whether to populate "versions" from git ranges.
   versions_from_repo = ndb.BooleanProperty(default=True)
+  # Ignore last import time once.
+  ignore_last_import_time = ndb.BooleanProperty(default=False)
   # HTTP link prefix.
   link = ndb.StringProperty()
   # DB prefix, if the database allocates its own.
@@ -665,7 +740,7 @@ class SourceRepository(ndb.Model):
 
     return False
 
-  def _pre_put_hook(self):
+  def _pre_put_hook(self):  # pylint: disable=arguments-differ
     """Pre-put hook for validation."""
     if self.type == SourceRepositoryType.BUCKET and self.editable:
       raise ValueError('BUCKET SourceRepository cannot be editable.')
@@ -687,7 +762,7 @@ def sorted_events(ecosystem, range_type, events):
   else:
     ecosystem_helper = ecosystems.get(ecosystem)
 
-  if ecosystem_helper is None:
+  if ecosystem_helper is None or not ecosystem_helper.supports_ordering:
     raise ValueError('Unsupported ecosystem ' + ecosystem)
 
   # Remove any magic '0' values.
