@@ -13,28 +13,49 @@
 # limitations under the License.
 """PURL conversion utilities."""
 
+from urllib.parse import quote
+
 PURL_ECOSYSTEMS = {
+    'Alpine': 'apk',
     'crates.io': 'cargo',
-    'Hex': 'hex',
+    'Debian': 'deb',
     'Go': 'golang',
+    'Hex': 'hex',
     'Maven': 'maven',
-    'NuGet': 'nuget',
     'npm': 'npm',
-    'Packagist': 'composer',
+    'NuGet': 'nuget',
     'OSS-Fuzz': 'generic',
+    'Packagist': 'composer',
+    'Pub': 'pub',
     'PyPI': 'pypi',
     'RubyGems': 'gem',
 }
 
 
+def _url_encode(package_name):
+  """URL encode a PURL `namespace/name` or `name`."""
+  parts = package_name.split('/')
+  return '/'.join(quote(p) for p in parts)
+
+
 def package_to_purl(ecosystem, package_name):
   """Convert a ecosystem and package name to PURL."""
   purl_type = PURL_ECOSYSTEMS.get(ecosystem)
   if not purl_type:
     return None
 
+  suffix = ''
+
   if purl_type == 'maven':
     # PURLs use / to separate the group ID and the artifact ID.
     package_name = package_name.replace(':', '/', 1)
 
-  return f'pkg:{purl_type}/{package_name}'
+  if purl_type == 'deb' and ecosystem == 'Debian':
+    package_name = 'debian/' + package_name
+    suffix = '?arch=source'
+
+  if purl_type == 'apk' and ecosystem == 'Alpine':
+    package_name = 'alpine/' + package_name
+    suffix = '?arch=source'
+
+  return f'pkg:{purl_type}/{_url_encode(package_name)}{suffix}'
