@@ -81,18 +81,26 @@ def _checkout_branch(repo, branch):
   repo.reset(remote_branch.target, pygit2.GIT_RESET_HARD)
 
 
-def clone(git_url, checkout_dir, git_callbacks=None):
-  """Perform a clone."""
-  # Use 'git' CLI here as it's much faster than libgit2's clone.
+def _set_git_callback_env(git_callbacks):
+  """Set the environment variable to set git callbacks for cli git"""
   env = {}
   if git_callbacks:
     env['GIT_SSH_COMMAND'] = (
         f'ssh -i "{git_callbacks.ssh_key_private_path}" '
         f'-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null '
         f'-o User={git_callbacks.username} -o IdentitiesOnly=yes')
+  return env
+
+
+def clone(git_url, checkout_dir, git_callbacks=None):
+  """Perform a clone."""
+  # Use 'git' CLI here as it's much faster than libgit2's clone.
+  env = _set_git_callback_env(git_callbacks)
 
   subprocess.check_call(
-      ['git', 'clone', _git_mirror(git_url), checkout_dir], env=env)
+      ['git', 'clone', _git_mirror(git_url), checkout_dir],
+      env=env,
+      stderr=subprocess.STDOUT)
   return pygit2.Repository(checkout_dir)
 
 
@@ -123,7 +131,12 @@ def _use_existing_checkout(git_url,
   repo = pygit2.Repository(checkout_dir)
   repo.cache = {}
   if repo.remotes['origin'].url != _git_mirror(git_url):
-    raise RuntimeError('Repo URL changed.')
+    # The URL in the code is the source of truth,
+    # so if the remote URL does not match
+    # update to the correct URL.
+    logging.warning('origin URL updated:\nOld: %s\nNew: %s',
+                    repo.remotes['origin'].url, _git_mirror(git_url))
+    repo.remotes['origin'].url = _git_mirror(git_url)
 
   if branch:
     _checkout_branch(repo, branch)
@@ -156,7 +169,11 @@ def ensure_updated_checkout(git_url,
 
 def reset_repo(repo, git_callbacks):
   """Reset repo."""
-  repo.remotes['origin'].fetch(callbacks=git_callbacks)
+  env = _set_git_callback_env(git_callbacks)
+  # Use git cli instead of pygit2 for performance
+  subprocess.check_call(['git', 'fetch', 'origin'], cwd=repo.workdir, env=env)
+  # Pygit2 equivalent of above call
+  # repo.remotes['origin'].fetch(callbacks=git_callbacks)
   remote_branch = repo.lookup_branch(
       repo.head.name.replace('refs/heads/', 'origin/'),
       pygit2.GIT_BRANCH_REMOTE)
